#!/usr/bin/env python
# encoding: utf-8

# MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>


"""Tools for reading out magma lists. 

Usage: magma_list.py CMD [OPT]

Commands: 
    - magnets_from_magma MAGMA_FILE
    - magma_from_files MAGMA FILE1 FILE2 ... 

Examples: 
    - magma_list.py magnets_from_magma example-0.4.magma
    - magma_list.py magma_from_files example-0.4.magma blah.txt foo.txt gam.txt
"""

__depends__ = "yaml"

try: # local install
	from create_simple_magma_list import FILE_LIST_NAME, URN_PARAMETER_NAME, MAGMA_0_4_HEADER, FILENAME_PARAMETER
except: # Installed version
	from magma.create_simple_magma_list import FILE_LIST_NAME, URN_PARAMETER_NAME, MAGMA_0_4_HEADER, FILENAME_PARAMETER


# We need yaml loading and dumping as main component. 
from yaml import load, dump

# Now we need the central magma class. This does most of the work. 

class Magma(object):
    """A Magma List."""
    def __init__(self, magma_file=None, input_files=None, creator=None, data=None, yaml_data=None, *args, **kwds):
        """A Magma list. 
        
        @param magma_file: The path to the magma file, either absolute or relative the the location this script is invoked from. 
        @type magma_file: String
        
        @param input_files: A list of file_paths of input files which should be included in the magma list. 
        @type input_files: List
        
        @param creator: An identifier for the creator of the Magma file. 
        @type creator: String
        
        @param yaml_data: Raw file data as read from a Magma file. 
        @type yaml_data: String
        
        @param data: A native Python dict with the data. 
        @type data: Dict
        
        """
        #: All data inside the Magma list
        self.data = None
        # if we get data, we assign it. 
        if data is not None: 
            self.data = data
        # If we get data in yaml format (String), we read it out. 
        elif yaml_data is not None: 
            self.data = self.readout_yaml(yaml_data)
        # If we get a magma file, we readout the file itself. 
        elif magma_file is not None: 
            self.data = self.readout_magma_from_file(magma_file)
        # If we get single files, we create the magma file from the files. 
        elif input_files is not None: 
            self.data = self.create_magma_from_files(file_paths=input_files)
        else: 
            raise Exception("No input specified. Youl need to pass one of data=DATA, magma_file=PATH or input_files=[PATH1, PATH2, ...]")
            
        # If we get a creator name, also add that. 
        if creator is not None: 
            self.data["creator"] = creator
        
        #: All files inside the Magma list. 
        self.files = self.files()
        
        #: All magnets inside the Magma list. 
        self.magnets = self.get_file_magnets()
        
        #: The metadata of this Magma list (excluding the files). 
        self.metadata = self.get_magma_metadata()
        
    def __str__(self): 
        """Return a human readable representation of the yaml file.
        
        In effect return the yaml data with the magic header.
        
        @return: A human readable representation of the Magma file in yaml format (String)."""
        # Then return its Magma representation (Magma Header + yaml data). 
        return self.dump()
        
    
    def remove_empty_entries(self): 
        """Remove empty entries in the Magma, for example empty alt-locs.
        
        We remove each entry by hand, because we should know which entries can be created by this program - and if we don't we know that have to change our coding practice :) ."""
        
        # Just tell every file to clean itself. 
        for i in self.files: 
            i.remove_empty_entries()
            
    def create_magma_from_files(self, file_paths): 
        """Create a magma file from given file paths. 
        
        @return: The Python dict for a Magma file - the magma data. 
        """
        try: 
            from create_simple_magma_list import Magma as SimpleMagma
        except: 
            from magma.create_simple_magma_list import Magma as SimpleMagma
        magma = SimpleMagma(file_paths)
        return magma.data
    
    def is_magma(self, yaml_data):
        """Check, if a given yaml data begins with the Magma header.
        
        @param yaml_data: Data in yaml format
        @type yaml_data: String
        @return: True or False
        """
        return yaml_data[:len(MAGMA_0_4_HEADER)] == MAGMA_0_4_HEADER
    
    def readout_yaml(self, yaml_data):
        """Readout a magma list from data in yaml format.
        
        @param yaml_data: Data in yaml format.
        @type yaml_data: String
        @return: The Python dict for a Magma file - the magma data. 
        """
        # First we now need to check, that this file is in MAGMAv0.4 format. 
        assert self.is_magma(yaml_data), "Doesn't begin with Magma v0.4 header " + MAGMA_0_4_HEADER
        # Now readout the data via yaml and return it. 
        data = load(yaml_data)
        # yaml returns None, if nothing is avaible. We need an empty dict though, if the Magma file was empty. 
        if data is None:
            data = {}
        return data
    
    def readout_magma_from_file(self, file_path):
        """Readout a magma list from the file_path
        
        @param file_path: The path to the magma file, either absolute or relative the the location this script is invoked from. 
        @type file_path: String
        @return: The Python dict for a Magma file - the magma data. """
        # First get the Magma file. This cries out, if the file doesn't exist. 
        file_handle = open(file_path, "r")
        # Read the data. 
        file_data = file_handle.read()
        # And close the file. 
        file_handle.close()
        # At the end, return the data. 
        return self.readout_yaml(file_data)
        
    def files(self):
        """Get the files from the magma data.
        
        @return: A list of all MagmaFile objects from the Magma.
        """
        # The files are held as simple list inside the magma file. 
        #: The list of files inside the Magma list. 
        file_list = []
        
        # If the Magma list contains files, we read them. 
        if FILE_LIST_NAME in self.data.keys(): 
            # We turn every file dict inside the magma list into a MagmaFile object and return them. 
            for i in self.data[FILE_LIST_NAME]: 
                file_list.append(MagmaFile(i))
        # Now the only thing left is returning the list. 
        return file_list
        
    def get_file_magnets(self): 
        """Get the magnet links of the files.
       
        # TODO: Include Alt-Locs. 

        @return: A list of all magnet links inside the Magma.
        """
        #: The list of magnets
        magnets = []
        # Add the magnet of each of the files. 
        for i in self.files: 
            magnets.append(i.magnet)
        return magnets
    
    def get_magma_metadata(self):
        """Get only the metadata, without the list of files.
        
        @return: A dict which contains only the metadata of the Magma without the files.
        """
        # First we need an empty dictionary to hold the metadata (we want to carve out the files key, so we just add all other keys). 
        magma_metadata = {}
        # Now we add all subdictionaries, except the "files" subdict. 
        for i in self.data: 
            if i != FILE_LIST_NAME: 
                magma_metadata[i] = self.data[i]
        # That's it. 
        return magma_metadata
    
    def magma_v0_2_filelist_representation(self):
        """@return: A list of files which can be read by MAGMAv0.2 compliant programs."""
        # First we get the list parameter. 
        data = "list:\n"
        
        # Then we append a magnet for each file. 
        for i in self.files: 
            data += ' - "' + i.magnet + '"\n'
        
        return data
    
    def magma_v0_2_data(self): 
        """@return: Data in Magma v0.2 format"""
        # First we begin with the MAGMAV0.2 header line. 
        data = "#MAGMAv0.2\n"
        
        # Then we append the file list. 
        data += magma_v0_2_filelist_representation()
        
        return data
        
        
    def save(self, path, magma_versions = [0.2, 0.4]):
        """Save the magma list to the path. 
        
        If the folder to save in doesn't exist, just raise an Exception.
        
        TODO: Clean out empty entries in the files and the metadata before saving. 
        self.data["gnutella"]["alt-locs"] = [] should disappear (if there are no other keys than alt-locs in gnutella, the gnutella dict should completely disappear, too). 

        @param path: The path to the target file.
        @type path: String
        
        @param magma_versions: The versions of the magma specification to be supported. 
        @type magma_versions: List of Floats
    
        @return: None
        """
        # First remove empty entries, so we get a nice looking file. 
        self.remove_empty_entries()
        # First get the file handle for the magma file
        file_handle = open(path, "w")
        # Then write the data in yaml format. 
        file_handle.write(self.dump(magma_versions=magma_versions))
        # And close the file handle. 
        file_handle.close()
        
    def print_data(self):
        """Print the magma file in the yaml format.
        
        @return: A string representation of the Magma in yaml format.
        """
        # Then print the data
        print self.__str__()
        
    def get_yaml_data(self):
        """Return the yaml representation of the data.
        
        @return: The yaml representation of the data."""
        return dump(self.data, default_flow_style=False)
        
    def dump(self, magma_versions = [0.2, 0.4]):
        """Return the yaml data. 
        
        @return: A string representation of the Magma in yaml format.
        """
        # If we want 0.2 support, we add it. 
        if 0.2 in magma_versions and 0.4 in magma_versions: 
            # But first we remove the former 0.2 list. 
            if "list" in self.data.keys(): 
                del(self.data["list"])
            # Now we add a new list from the data we got. This can REMOVE DEPRECATED DATA. 
            return "#MAGMAv0.4\n" + self.get_yaml_data() + "\n# Magmav0.2 compatibility section\n" + self.magma_v0_2_filelist_representation()
        elif 0.4 in magma_versions: 
            return "#MAGMAv0.4\n" + self.get_yaml_data()


# Also we have files inside the magma list. They hate a few extra attributes to be easier to use than raw dictionaries. 

class MagmaFile(object):
    """A file inside the magma list.
    
    Provides some abstractions for files inside Magma objects. 
    
        >>> magma_file = MagmaFile({FILENAME_PARAMETER: "input_file.txt", "urn": {"sha1": "3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G"}})
        >>> print magma_file.data
        {'urn': {'sha1': '3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G'}, 'Filename': 'input_file.txt'}
        >>> print magma_file.sha1_hash
        3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G
        >>> print magma_file.name
        input_file.txt
        >>> print magma_file.magnet
        magnet:?xt=urn:sha1:3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G&dn=input_file.txt

    
    """
    
    def __init__(self, data, *args, **kwds):
        """A file inside the magma list. 
        
        @param data: The data of this file. 
        @type data: Dict
        """
        #: All data of the file
        self.data = data
        #: The name of the file, so we can easily print it. Every MagmaFile needs a filename. 
        self.name = self.data[FILENAME_PARAMETER]
        #: The sha1 hash of the file. Every MagmaFile needs a sha1 hash. 
        self.sha1_hash = self.data[URN_PARAMETER_NAME]["sha1"]
        #: The list of alternate urls. 
        self.urls = self.readout_urls()
        #: The Other Gnutella clients who have the file. 
        self.Gnutella_alt_locs = self.readout_gnet_alt_locs()
        
    
    def __str__(self): 
        """Show nicely readable information about the file.
        
        @return: Nicely readable information about the file (String). """
        return self.name + " " + self.sha1_hash

    def readout_urls(self):
        """Readout alternate urls from the data
        
        @return: A list of URLs which can be used as alternate fallback sources (which are unsafe, though, since it isn't sure that they will point to the same file when times change). """
        # If we have urls, return them. 
        if "urls" in self.data.keys(): 
            return self.data["urls"]
        # Else return an empty list. 
        else: 
            # First create the list, so adding new urls gets saved in the file. 
            self.data["urls"] = []
            # And return the empty list. 
            return self.data["urls"]


    def readout_gnet_alt_locs(self):
        """Readout the Alt-Locs from the data
        
        @return: A list of IPs with Port numbers which can be used together with the sha1 hash to construct alt-locs for Gnutella. """
        ALT_LOC_STRING = "alt-locs"
        # If we have alt-locs, return them. 
        if "gnutella" in self.data.keys() and ALT_LOC_STRING in self.data["gnutella"].keys(): 
            return self.data["gnutella"][ALT_LOC_STRING]
        # Else return an empty list. 
        else: 
            # First create the list, so adding new alt-locs gets saved in the file. 
            if not "gnutella" in self.data.keys(): 
                self.data["gnutella"] = {}
            self.data["gnutella"][ALT_LOC_STRING] = []
            # And return the empty list. 
            return self.data["gnutella"][ALT_LOC_STRING]
        
    
    def parse_magnet(self): 
        """Parse the magnet link from the data inside the file.
        
        @return: A magnet link for the file (String). """
        # From this we create the xt. 
        xt = "urn:sha1:" + self.sha1_hash
        # And from it we get dn
        dn = self.name
        
        # Now we get our url parameters. 
        params = [("xt", xt ), ("dn", dn )]
        
        # Additionally we want to add alt-locs. 
        gnet_alt_locs = self.readout_gnet_alt_locs()
        # They need to be transformed to the correct xs parameter, though. 
        for i in gnet_alt_locs: 
            params.append(("xs", "http://" + str(i) + "/uri-res/N2R?" + xt))
            
        
        # Now get the urlencode method
        from urllib import urlencode
        #: The magnet link is a String. 
        magnet = "magnet:?" + urlencode(params)
        # TODO: Include alt-locs. 
        return magnet
    
    #: The magnet link of the file
    magnet = property(fget=parse_magnet, fset=setattr)
    
    def remove_empty_entries(self): 
        """Remove empty entries from the file, for example empty alt-locs."""
        # Check each key, if it points to an empty dictionary, or only to empty subelements. 
        
        # First check for empty subkeys. 
        for i in self.data: 
            try: 
                # Iterate over the items inside the key
                for j in self.data[i].keys(): 
                    try: 
                        # if it has a length and the length is 0, we remove it. 
                        if len(self.data[i][j]) == 0: 
                            del(self.data[i][j])
                    except: pass
            except: pass
        
        # Now  check for empty keys. 
        for i in self.data.keys(): 
            try: 
                # If the length of the thing is 0, the whatever-it-is empty and we can remove it. 
                if len(self.data[i]) == 0: 
                    del(self.data[i])
            except: # If we can't get a length, we just ignore it. It seems to be data/content, not structure. 
                pass
        #TODO: Make this rekursive, so we can clean arbitrary deeply nested structures. 

def usage(): 
    """Usage instructions.
    
    @return: Usage instructions (String)."""
    # Just use the docstring. 
    usage = __doc__.split("\n\n")[1:]
    usage_string = "\n\n".join(usage)
    return usage_string
    
def _test(): 
    """Do all doctests.
    
    @return: None"""
    from doctest import testmod
    testmod()

# If this gets called automatically, load the magma list and print the magnets, or create a magma list. 
if __name__ == "__main__": 
    # _test()
    from sys import argv
    if len(argv) < 3: 
        print usage()
    elif argv[1] == "magnets_from_magma": 
        magma = Magma(magma_file=argv[2])
        for i in magma.magnets: 
            print i
    elif argv[1] == "magma_from_files": 
        magma = Magma(input_files=argv[3:])
        magma.save(argv[2])
    else: 
        print usage()



#!/usr/bin/env python
# encoding: utf-8

# MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

"""Create magma files interactively via commandline dialogs.

Usage: magma_creator_cli.py file1 file2, ...

Examples: 
    - magma_creator_cli.py input_file.txt input_file2.txt Infinite-Hands--free-software.ogg 


This will implement features from magma-for-streaming-mediav0.07.txt

It is mainly a proof of concept, yet, and it may change! 

Ideas: 
    - Check, if tagpy or similar are avaible, and if yes, add metadata from the file tags. 
"""

# Import basic Magma classes. 
from magma_list import Magma
from magma_list import MagmaFile 


def edit_file(magma_file):
    """Edit the data inside a magma file - add metadata."""
    # First check, which metadata we can get automatically. 
    print "First this script will try to get some values automatically. "
    automatically_generated_tags = []
    
    if not magma_file.name.endswith(".mp3") or magma_file.name.endswith(".ogg"): 
        print "No mp3 or ogg file. We only support mp3 and ogg vorbis at the moment."
        return 
    
    # Create the audio subdict
    magma_file.data["audio"] = {}
    try: 
        from tagpy import FileRef
        # The basic object from tagpy. 
        filetags = FileRef(magma_file.path)
        print "yes, FileRef"
        # The object with audiodata. 
        try: 
            audioprops = filetags.audioProperties()
        except: pass
        try: 
            tagfile = filetags.file()
        except: pass
        try: 
            tag = filetags.tag()
        except: pass
        
        try: 
            # The length of time this file plays. 
            magma_file.data["audio"]["length-ms"] = audioprops.length*1000
            automatically_generated_tags.append(("length-ms", magma_file.data["audio"]["length-ms"]))
        except: pass
        try: 
            # The bitrate of the file. 
            magma_file.data["audio"]["bitrate-kbps"] = audioprops.bitrate
            automatically_generated_tags.append(("bitrate-kbps", magma_file.data["audio"]["bitrate-kbps"]))
        except: pass
        try: 
            # The sampling rate. 
            magma_file.data["audio"]["samplerate-Hz"] = audioprops.sampleRate
            automatically_generated_tags.append(("samplerate-Hz", magma_file.data["audio"]["samplerate-Hz"]))
        except: pass
        try: 
            # And the raw filesize. 
            magma_file.data["size-Bytes"] = tagfile.length()
            automatically_generated_tags.append(("size-Bytes", magma_file.data["size-bits"]))
        except: pass
        
        # Now to the general metatags of the file content. 
        try: 
            # The title of the file
            magma_file.data["audio"]["title"] = tag.title
            automatically_generated_tags.append(("title", magma_file.data["audio"]["title"]))
        except: pass
        
        try: 
            # The publish year of the file
            magma_file.data["audio"]["year"] = tag.year
            automatically_generated_tags.append(("year", magma_file.data["audio"]["year"]))
        except: pass
        
        try: 
            # The track number of the file
            magma_file.data["audio"]["track"] = tag.track
            automatically_generated_tags.append(("track", magma_file.data["audio"]["track"]))
        except: pass
        
        try: 
            # The genre of the file
            magma_file.data["audio"]["genre"] = tag.genre
            automatically_generated_tags.append(("genre", magma_file.data["audio"]["genre"]))
        except: pass
        
        try: 
            # The comment of the file
            magma_file.data["audio"]["comment"] = tag.comment
            automatically_generated_tags.append(("comment", magma_file.data["audio"]["comment"]))
        except: pass
    except: pass
    
    print "The following tags were added:"
    for i in automatically_generated_tags: 
        print i
    
    



def create_magma_list_interactively(files):
    """Create magma files interactively via commandline dialogs.
    
    @param files: A list of files to hash.
    @type files: List
    """
    magma = Magma(input_files=files)
    print "Basic magma list: "
    print magma
    
    print "------ ------ ------ ------ ------ ------ "

    print "Now we try add metadata using tagpy. "
    for i in range(len(magma.files)): 
        magma.files[i].path = files[i]
        edit_file(magma.files[i])
    
    print "------ ------ ------ ------ ------ ------ "
    print "The magma list now looks like this: "
    print "---"
    print magma
    
    ip_port = raw_input("... \nIf you use Gnutella, you can now give a list of IP:PORT pairs seperated by spaces as alternate-locations. For example you can give your clients IP and its port, alternately your dyndns host and your client port. They can greatly increase the performance of downloads but break your anonymity. \nExample: 127.0.0.1:6465 0.0.0.0:9999 example.dyndns.org:1234 \nIf you want to add alt-locs, please add the IP:PORT pairs now: ")
    alt_locs = ip_port.split()
    
    try: 
        #from IPy import IP
        #for i in alt_locs: 
        #    ip = IP(i.split(":")[0])
    
        if len(alt_locs) > 0: 
            for i in magma.files: 
                i.data["gnutella"]["alt-locs"] = alt_locs
    except: 
        print "You need the IPy module to add alt-locs, and the IP addresses need to be valid. No alt-locs added."
        print "To fix it just run 'easy_install IPy', and check the format of the IPs"
    
    target_file = raw_input("Where shall we save it? ")
    magma.save(target_file)
    
    

def usage(): 
    """Usage instructions.
    
    @return: Usage instructions (String)."""
    # Just use the docstring. 
    usage = __doc__.split("\n\n")[1:]
    usage_string = "\n\n".join(usage)
    return usage_string

#### Self-Test ####

def _test(): 
    """Do all doctests.
    
    @return: None"""
    from doctest import testmod
    testmod()

# If this gets called automatically, load the magma list and print the magnets, or create a magma list. 
if __name__ == "__main__": 
    _test()
    from sys import argv
    if argv[1] in ["--help", "-h", "?", "--usage"]: 
        print usage()
    else: 
        create_magma_list_interactively(argv[1:])



#!/usr/bin/env python
# encoding: utf-8

# MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

"""
Returns a magma-list containing the file name. Takes the names as input.

# Test strip_dirname

    >>> a = "blah/blubb/bangala/goi.gis"
    >>> print strip_dirname(a)
    goi.gis
    
    >>> a = "/blah/blubb/bangala/goi.gis"
    >>> print strip_dirname(a)
    goi.gis
    
"""

__depends__ = 'yaml, os'



#: The name of the file list parameter inside the Magma file.
FILE_LIST_NAME = "files"
#: The name of the dict holding the hashes and other unique identifiers. 
URN_PARAMETER_NAME = "urn"
#: The header (magic line) of Magma v0.4 files. 
MAGMA_0_4_HEADER = "#MAGMAv0.4"

#: The name of the filename parameter. 
FILENAME_PARAMETER = "filename"


# Get the sha1_gnutella module to create a sha1 for Gnutella. 
from sha1_gnutella import sha1_gnutella
# And get the yaml dumper and loader
from yaml import dump, load
# And the dirname to be able to strip it. 
from os.path import dirname

def strip_dirname(path):
    """Strip everything but the name of the file.
    
    @param path: A path to a file. 
    @type path: String
    @return: The name of the file, stripped of anything before it. 
        
        Example: /home/blah/blubb.magma becomes
        
        blubb.magma
       """
    # We remove just as many characters from the beginning of the string as the dirname is long. 
    if len(dirname(path)) > 0: 
        filename = path[len(dirname(path)) + 1:]
    else: 
        filename = path
    return filename

class Magma: 
    """A Simple Magma object created from a list of input file paths."""
    def __init__(self, inputfilenames, *args, **kwds):
        """A Simple Magma object created from a list of input file paths.
        
        @param inputfilenames: The paths to the files to hash.
        @type inputfilenames: List
        """
        #: The names of the files. 
        self.filenames = inputfilenames
        #: Sha1 values to the files. The sha1 gets computed for each one here, so this isn't ambigous. 
        self.sha1 = {}
        # Add sha1 hashs for each file. 
        for i in self.filenames: 
            self.sha1[i] = sha1_gnutella(i)
        #: The raw python data of the Magma file. 
        self.data = self.data()
        #: The yaml representation. 
        self.magma = self.magma()
        
    def data(self):
        """Get the Python data of the Magma file (a dict).
        
        @return: The data of the Magma file as Python dict. """
        magma = {}
        list_of_magnets = []
        for i in self.filenames: 
            magnet = {}
            magnet[URN_PARAMETER_NAME] = {}
            magnet[URN_PARAMETER_NAME]['sha1'] = self.sha1[i]
            magnet[FILENAME_PARAMETER] = strip_dirname(i)
            list_of_magnets.append(magnet)
        magma[FILE_LIST_NAME] = list_of_magnets
        return magma
        
    def magma(self): 
        """Dump the yaml representation of the file with a MAGMA header as string.
        
        @return: A string representation of the Magma file (in yaml format). """
        return MAGMA_0_4_HEADER + "\n" + dump(self.data, default_flow_style=False)

def _test(): 
    """Do all doctests
    
    @return: None
    """
    from doctest import testmod
    testmod()

#### Self-Test ####
if __name__ == '__main__': 
    from sys import argv
    if len(argv) > 1: 
        filenames = argv[1:]
    else: 
        # filename = "magma_liste_erzeugen.py"
        filenames = [raw_input("Dateiname und Pfad eingeben: ")]
        gnutella_client_ip = raw_input("Eigene IP oder dyndns-Adresse eingeben: ")
    magma = Magma(filenames)
    print magma.magma
    
    # And test. 
    _test()
    



#!/usr/bin/env python
# encoding: utf-8

# MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>


"""Sha1 Gnutella - Readout files and calculate the sha1-hashes. Output them base32 encoded. 

Usage: 
 - sha1_gnutella.py <file1> <file2> ... 


Output: 
<sha1-hash1> <filename1>
<sha1-hash2> <filename2>
""" 

#### Background ####

__depends__ = 'hashlib, base64'

#### Background ####

#### Imports ####

# We need the sha1-hashing function from haslib
from hashlib import sha1

# and for encoding the hash we need the base32 encoding function from base64
from base64 import b32encode

from os.path import isfile

#### Imports ####

#### Constants ####

#: The length of the segments to read in one go. 
FILE_FRAGMENT_SIZE_BYTES =  10 * 1024 * 1024 # 10 MiB, because it doesn't really make a difference, but like this, the disk might be able to do something else for a moment, if the files are bigger than 10MiB, and smaller files get accessed only once. 

#### Constants ####


def sha1_gnutella(filepath, fragment_size = FILE_FRAGMENT_SIZE_BYTES): 
    """Calculate the sha1 hash and output it base32 encoded as is required by magnet links. 
    
    @param filepath: The path to the file to hash. Must be a file! This function is too low level to add recursion into subdirectories. 
    @type filepath: String
    
    @param fragment_size: Length of the fragment to read in bytes.
    @type fragment_size: Int
    
    @return: The sha1 hash, base32 encoded (String). 
    """
    
    # As really first part, we check, if the path leads to a file. If it doesn't we just return. 
    if not isfile(filepath): 
        raise Exception("Error when reading file. Does the path exist? " + filepath)
    
    # First we create 
    #: The file-object
    try: 
        file = open(filepath, "r")
    except: 
        print "Error when reading file. Does the path exist?", filepath
        raise Exception("Error when reading file. Does the path exist? " + filepath)
    # Then we get 
    #: A line of data readout from the file. 
    fragment = file.read(fragment_size)
    
    # so we can calculate arbitrary large files without running out of memory. 
    
    # Next we call in
    #: The sha1-function
    sha = sha1()
    
    # Into which we can then route the data. 
    
    # If the fragment is smaller than the fragment_size, 
    if len(fragment) < fragment_size and False: 
        # we can close the file instantly 
        # and then get the sha1. 
        # This way we get rid uf uselessly open files at once. 
        file.close()
        sha.update(fragment)
    else: 
        # while the fragment isn't empty
        # We put the fragments into sha1. 
        
        # We first update once
        sha.update(fragment)
        
        # And then only update as long as the length of the fragment is equal to the fragment size. 
        # If we'd test, if it's empty, we'd need one useless read. 
        while len(fragment) == fragment_size: 
            # Get the next fragment
            fragment = file.read(fragment_size)
            # And update the sha1
            sha.update(fragment)
        file.close()
    # Via digest, this function can then output
    # the sha1 hash as bytes. 
    # we put it through bb32encode() to generate
    #: The base32 encoded sha1-hash
    xt_sha1 = b32encode(sha.digest())
    
    # At last return the base32 encoded sha1 hash of the file. 
    return xt_sha1

#### User-Input ####

# This sample input just calculates the sha1-hash of the script itself. 

filepath = "sha1_gnutella.py" 
# filepath = raw_input("Dateiname und Pfad eingeben: ")

#### User-Input ####

#### Self-Test ####

def test_speed(files, min=1024 * 1024, max=15*1024*1024, step=1024 * 1024):
    """Test different settings of the fragment_size. 
    
    Result: The fragment_size is almost irrelevant. Anything between 512kiB and 10MiB offers about the same speed.
    
    @param files: The files to hash.
    @type files: List
    @return: None
    """
    from time import time
    start_time = 0
    stop_time = 0
    for i in range(min, max, step): 
        start_time = time()
        for j in files: 
            try: 
                sha1_gnutella(j, i)
            except: pass
        stop_time = time()
        print "Segment_Size (MiB):", i / 1024.0 / 1024.0, "time:", stop_time - start_time

def help(): 
    """Display the usage information."""
    print __doc__

# If the script is called directly 
# and given a commandline argument, 
# parse that file. 

if __name__ == "__main__": 
    from sys import argv
    if len(argv) < 2 or argv[1] in ["-h", "--help", "?"]: 
        help()

    else: 
        # Calculate a sha1 for all given files.
        for i in argv[1:]: 
            try: 
                print sha1_gnutella(i), i
            except: pass

### Self-Test ####



#!/usr/bin/env python
# encoding: utf-8

# MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>


"""MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 

Magma lists are lists of files which can be downloaded via magnet links. 

They are written in yaml format to be easily readable as well as flexible and powerful. 


A simple Magma file looks like the following:: 
    #MAGMAv0.4
    files:
    - filename: input_file.txt
      urn:
        sha1: 3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G
    - filename: input_file2.txt
      urn:
        sha1: 3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G


This script parses the basic features of MAGMAv0.4, which is being specified at the moment. 
It doesn't parse the current MAGMAv0.2 files!

Use it only, if you want to take a peek at what the updated specification will bring. 


It depends on U{pyyaml <http://pyyaml.org/>}, tagpy and IPy. 

An example magma file is I{example-0.4.magma}. 

This API documentation is avaible from U{http://gnuticles.gnufu.net/pymagma/}, and the code is avaible from a U{Mercurial repository <http://freehg.org/u/ArneBab/magma/>}. 

The module can also be found in the U{Python Package Index (PyPI) <http://pypi.python.org/pypi/magma/>}. It is available under the GPL-3 or later. 

It is being written by U{Arne Babenhauserheide <http://draketo.de>}. If you wish to contribute, please drop me a mail with the word "tunnel" somewhere in the subject line (to get past my spamfilter). My email: arne_bab at web de . 

Usage of this module
====================

    >>> # from magma import *
    >>> magma = open_file("example-0.4.magma")

or 

    >>> magma = create_from_input_files(["input_file.txt", "input_file2.txt"])


API / usage
===========

B{Creating magma lists from input files}
    
Create a list from input files. 
    >>> magma = create_from_input_files(["input_file.txt"])
    
Output the list, showing the empty entries, which could be filled with data. 
    >>> print magma
    #MAGMAv0.4
    files:
    - filename: input_file.txt
      gnutella:
        alt-locs: []
      urls: []
      urn:
        sha1: 3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G
    <BLANKLINE>
    # Magmav0.2 compatibility section
    list:
     - "magnet:?xt=urn%3Asha1%3A3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G&dn=input_file.txt"
    <BLANKLINE>
    
And save it to a file with the empty entries cleaned away. 
    >>> magma.save(path="example.magma")


Or just return a list of its files as MagmaFiles. 
    >>> files = magma.files
    
Or output all magnet links inside the magma list. 
   >>> print magma.magnets
   ['magnet:?xt=urn%3Asha1%3A3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G&dn=input_file.txt']
  
  
B{Reading from magma lists}
   
Readout the data from a file. 
   >>> magma = open_file("example.magma")
   
Now get the files inside the list. 
   >>> files = magma.files
   >>> for i in files: print i
   input_file.txt 3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G
   
Get the first file from the list. 
   >>> one_file = files[0]
   >>> print one_file
   input_file.txt 3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G
   
And output its magnet. 
   >>> print one_file.magnet
   magnet:?xt=urn%3Asha1%3A3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G&dn=input_file.txt
   
Or output all its (meta-) data. 
   >>> print one_file.data
   {'urn': {'sha1': '3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G'}, 'gnutella': {'alt-locs': []}, 'urls': [], 'filename': 'input_file.txt'}
   

Output all magnet links inside the magma list. 
   >>> print magma.magnets
   ['magnet:?xt=urn%3Asha1%3A3UJCLAOIZVCNAIT7TQYFLAP7ZNFW6G2G&dn=input_file.txt']

Load a Magma object from a string containing Magma data via magma.load(). 
As test, load a minimal Magma file, containing nothing but the header.
   >>> magma = load("#MAGMAv0.4")
   
And cry out, if the string doesn't begin with the Magma header (isn't Magma data!). 
   >>> magma = load("a:b")
   Traceback (most recent call last):
   ...
   AssertionError: Doesn't begin with Magma v0.4 header #MAGMAv0.4

Also dump yaml data via magma.dump()
   >>> print dump(magma)
   #MAGMAv0.4
   {}
   <BLANKLINE>
   # Magmav0.2 compatibility section
   list:
   <BLANKLINE>

Or just print the Magma data. 
   >>> print magma
   #MAGMAv0.4
   {}
   <BLANKLINE>
   # Magmav0.2 compatibility section
   list:
   <BLANKLINE>


Plans: Gnutella
    - Also pass metadata and include that alongside the file list. 
    - Add Alt-locs (depends on the network used). Maybe GNet-Alt-Loc for Gnutella. Gets parsed to magnet xs automatically. 
    - Alt-locs at ["sources"]["gnutella"]["alt-locs"] instead of ["gnutella"]["alt-locs"] -> general sources possible. list of HTTP sources at: ["sources"]["http"]
   
Ideas: 
    - Readout metadata from files, for example using the taglib, pyPdf or such. 
    - If given both magma list and files, the magma list gets extended by the files. 
    - Joining and seperating magma lists. 
    - Passing several additional input lists to join with the main list. 
    - Passing the data of files along with magma metadata to create the list. 
    - Passing the indizes of files to automatically slice the list. maybe even with the syntax sliced_magma = magma[a:b]
    - If a dir gets passed, add all files within that dir. Alsa a parameter to ask, if it should be done recursively. 
    - If hashing files takes very long (for example seen with clock.tick()), give more output. 
    - Add more commandline switches (verbose, help, ... - see the babtools_gentoo script)

"""

#### Background ####

__author__ = 'Arne Babenhauserheide'
__copyright__ = 'Copyright (C) 2008 Arne Babenhauserheide'
__date__ = '2008-05-09'
__licence__ = 'GPL-3 or later'
__program__ = 'MAGnet MAnifest Management'
__version__ = '0.3.9'

# The dependencies of the _init__ only. 
__depends__ = ''

def parse_dependencies(init_depends):
    """Aggregate the dependencies from all submodules.
    
    @param init_depends: The dependencies of the __init__ file. 
    @type init_depends: String 
    @return: A string with dependencies, seperated by ', '. 
    """
    # aggregate dependencies from all submodules. 
    dep_list = [init_depends]
    from create_simple_magma_list import __depends__ 
    dep_list.append(__depends__)
    from magma_list import __depends__
    dep_list.append(__depends__)
    from sha1_gnutella import __depends__
    dep_list.append(__depends__)
    
    # Now aggregate it, simply using a set, which discards multiple entries. 
    deps = set()
    for i in dep_list: 
        # We need to turn the individual __depends__ into lists to add them. 
        [deps.add(j) for j in i.split(", ")]
    # And discard a possibly included emtpy string (if there is no "", nothing happens). 
    deps.discard("")
    
    # Now assign it as string to the __depends__
    return ", ".join(deps)

__depends__ = parse_dependencies(__depends__)


#### Background ####

#### Constants ####

#### Constants ####


from magma_list import Magma

def open_file(filepath):
    """Open a file as Magma list and produce the corresponding Python object.
    
    @param filepath: The path to a Magma file.
    @type filepath: String
    @return: A Magma Object. 
    """
    return Magma(magma_file=filepath)

def create_from_input_files(files):
    """Create a magma file from given input files.
    
    @param files: A list of file paths
    @type files: List of Strings
    @return: A Magma Object."""
    return Magma(input_files=files)

def load(yaml_data):
    """Parse a Magma list and produce the corresponding Python object.
    
    @param yaml_data: Data read from a yaml file
    @type yaml_data: String
    @return: A Magma Object. 
    """
    return Magma(yaml_data=yaml_data)
    
def dump(magma):
    """Return the string representation of the Magma file.
    
    @param magma: A Magma object.
    @type magma: magma.Magma
    @return: A String represenation of the Magma file (in yaml format). 
    """
    return magma.__str__()
    
    


#### Self-Test ####

def _test():
    """Do all doctests.
    
    @return: None
    """
    # Get the testmod function from the doctest module. 
    from doctest import testmod
    # And call it. 
    testmod()

# If this script gets called directly, do the doctests. 
if __name__ == "__main__": 
    _test()



#!/usr/bin/env python
# encoding: utf-8

# MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

"""Create the magma lists for a Phex release via the commandline. 

Usage: create_phex_release_magma.py <windows-release-path> <osx-release-path> <gnu/linux-file-release-path> 

Examples: 
    - create_phex_release_magma.py phex_3.2.2.104.exe phex_3.2.2.104_osx.zip phex_3.2.2.104.zip

Plans: 
    - Use MAGMAv0.4
    - Do it less hackish: Use the magma module. 

Ideas: 
    - Get list of Sourceforge servers autmatically. 
"""

### Constants ###

SOURCEFORGE_MIRRORS = ["http://heanet.dl.sourceforge.net/sourceforge/", "http://internap.dl.sourceforge.net/sourceforge/", "http://mesh.dl.sourceforge.net/sourceforge/", "http://puzzle.dl.sourceforge.net/sourceforge/", "http://easynews.dl.sourceforge.net/sourceforge/", "http://osdn.dl.sourceforge.net/sourceforge/", "http://keihanna.dl.sourceforge.net/sourceforge/", "http://optusnet.dl.sourceforge.net/sourceforge/", "http://switch.dl.sourceforge.net/sourceforge/", "http://ovh.dl.sourceforge.net/sourceforge/", "http://kent.dl.sourceforge.net/sourceforge/", "http://osdn.dl.sourceforge.net/sourceforge/", "http://jaist.dl.sourceforge.net/sourceforge/", "http://ufpr.dl.sourceforge.net/sourceforge/", "http://umn.dl.sourceforge.net/sourceforge/", "http://nchc.dl.sourceforge.net/sourceforge/", "http://jaist.dl.sourceforge.net/sourceforge/"] #: A list of SF.net servers. The name of the project and the name of the file get appended at the end (MIRROR + project/ + filename). 

#: The footer line for the phex update magma list. 
PHEX_MAGMA_FOOTER = '''

# Phex: Copyright 2006 The Phex Team - GPL-2 or later.
# Link: http://phex.org
# Developer-link: http://sf.net/projects/phex

'''


def create_phex_magmas(files):
    """Create the magma files for all release types. """
    
    #: The different target platforms / release types in the same order as the filepath parameters. 
    RELEASE_TYPES = ["win", "osx", "other"]
    
    # Create a magma file for each release. 
    for i in range(len(RELEASE_TYPES)): 
        create_phex_magma(files[i], release_type = RELEASE_TYPES[i])
    

def create_phex_magma(path, release_type="win"):
    """Create a magma list for a single release_type"""
    # We need to create the sha1 hash in Gnutella style. 
    from magma.sha1_gnutella import sha1_gnutella
    sha1 = sha1_gnutella(path)
    
    # Also we need to strip the dirname. 
    from magma.create_simple_magma_list import strip_dirname
    filename = strip_dirname(path)
    
    # Now, after we got all necessary data, we simply write the respective magma file. 
    write_magma(sha1, filename, release_type)
    
    # Done :) 
    
    

def write_magma(sha1, filename, release_type="other"):
    """Create the magma file for a specific release type. 
    
    @param release_type: The target platform: win, osx or other
    @type release_type: String """
    
    # First get the data to write. 
    
    # First the header line. This won't change but it is different for each release type. 
    
    if release_type == "osx": 
        magma_data = """#MAGMAv0.2 magnet:?mt=.&as=http://dateien.draketo.de/phex_osx.magma&dn=phex_osx.magma"""
    elif release_type == "win": 
        magma_data = """#MAGMAv0.2 magnet:?mt=.&as=http://dateien.draketo.de/phex_win.magma&dn=phex_win.magma"""
    elif release_type == "other": 
        magma_data = """#MAGMAv0.2 magnet:?mt=.&as=http://dateien.draketo.de/phex_win.magma&dn=phex_win.magma"""
    else: 
        raise Exception("Unsupported release type. Supported types are osx, win, other")
    
    # Now the middle part between the magma header and the phex magnet. 
    
    magma_data += """

#This list contains the update URLs for Phex. 

list:
  
 - """
    
    # Now the magnet. We need the SHA1 and the filename. 
    
    SHA1 = sha1
    
    
    # The first line isn't completely generic. 
    magma_data += '''"magnet:?xt=urn:sha1:''' + SHA1 + '''
   &dn=''' + filename
   
    # The next sources are plain Phex clients with stable IPs or dyndns hostnames. 
   
    GNET_CLIENT_SOURCES = ["http://edrikor.dyndns.org:9846/uri-res/N2R?urn:sha1:"]
    for i in GNET_CLIENT_SOURCES: 
        magma_data += "&xs=" + i + SHA1
    
    # Now add the freebase cache. 
    
    magma_data += "&as=http://www.freebase.be/g2/dlcount.php?sha1=" + SHA1
     
    # Finally add Sourceforge mirrors. 
    
    for i in SOURCEFORGE_MIRRORS: 
        magma_data += "&as=" + i + "phex/" + filename
    
    # Now close the magnet link
    
    magma_data += '''"'''
    
    # And add filename information. 
    magma_data += '''
  file-name: ''' + filename
 
    # Now finish the file. 
    magma_data += PHEX_MAGMA_FOOTER
    
    # Now write the file. 
    if release_type == "osx": 
        write_to_file("phex_osx.magma", magma_data)
    elif release_type == "win": 
        write_to_file("phex_win.magma", magma_data)
    elif release_type == "other": 
        write_to_file("phex_other.magma", magma_data)
    else: 
        raise Exception("Unsupported release type. Supported types are osx, win, other")



def write_to_file(path, data):
    """Write the data to a file at the path."""
    f = open(path, "w")
    f.write(data)
    f.close()
    log("wrote " + path)
    

def log(data, log_type = "console"):
    """Log some data."""
    if log_type == "console": 
        print data

    

def usage(): 
    """Usage instructions.
    
    @return: Usage instructions (String)."""
    # Just use the docstring. 
    usage = __doc__.split("\n\n")[1:]
    usage_string = "\n\n".join(usage)
    return usage_string

#### Self-Test ####

def _test(): 
    """Do all doctests.
    
    @return: None"""
    from doctest import testmod
    testmod()

# If this gets called automatically, load the magma list and print the magnets, or create a magma list. 
if __name__ == "__main__": 
    _test()
    from sys import argv
    if argv[1] in ["--help", "-h", "?", "--usage"]: 
        print usage()
    else: 
        create_phex_magmas(argv[1:])



#!/usr/bin/env python
# encoding: utf-8

# MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

"""Create magma files interactively via commandline dialogs.

Usage: magma_creator_cli.py file1 file2, ...

Examples: 
    - magma_creator_cli.py input_file.txt input_file2.txt Infinite-Hands--free-software.ogg 


This will implement features from magma-for-streaming-mediav0.07.txt

It is mainly a proof of concept, yet, and it may change! 

Ideas: 
    - Check, if tagpy or similar are avaible, and if yes, add metadata from the file tags. 
"""

# Import basic Magma classes. 
from magma_list import Magma
from magma_list import MagmaFile 


def ask_audio(files, magma): 
    """Ask, whether one of the files is an audio file."""
    print "Now, let's make it a audio list."
    print ""
    audio = raw_input("Is one of the file a audio file (yes/No)? ")
    # Ask until the user 
    while not audio.lower() in ["yes", "no", "", "y", "n"]: 
        audio = raw_input("Is one of the file a audio file? Please answer either 'yes' or 'no'. Just clicking enter skips: ")
    # If we have no audio files, we return instantly. 
    if audio.lower() == "no" or audio.lower() == "n" or audio.lower() == "": 
        return 
    
    # Else, the answer was yes, and we can go on. 
    # First get the file which is a audio file. TODO: Add support for multiple mutimedia files. 
    
    audio = raw_input("Please enter which file it is (1 or 2 or ...): ")
    while not audio in [str(i) for i in range(1, len(magma.files) + 1)]: 
        audio = raw_input("Please enter a valid number (1 or 2 or ...): ")
    print "Now onwards to editing file", audio + ":", magma.files[int(audio) -1].name
    file_to_edit = magma.files[int(audio) -1]
    file_to_edit.path = files[int(audio) -1]
    edit_file(file_to_edit)
    
    print "So now, the file looks like this:"
    
    print magma
    
    magma_filename = raw_input("Please enter a name to which to save the file. It should end on .magma: ")
    magma.save(magma_filename)
    
    print "Magma file saved as", magma_filename
    
    print "It contains the following magnets:"
    for i in magma.magnets: 
        print i


def edit_file(magma_file):
    """Edit the data inside a magma file - add metadata."""
    # First check, which metadata we can get automatically. 
    print "First this script will try to get some values automatically. "
    automatically_generated_tags = []
    # Create the audio subdict
    magma_file.data["audio"] = {}
    try: 
        from tagpy import FileRef
        # The basic object from tagpy. 
        filetags = FileRef(magma_file.path)
        # The object with audiodata. 
        audioprops = filetags.audioProperties()
        tagfile = filetags.file()
        try: 
            # The length of time this file plays. 
            magma_file.data["audio"]["length-ms"] = audioprops.length*1000
            automatically_generated_tags.append(("length-ms", magma_file.data["audio"]["length-ms"]))
        except: pass
        try: 
            # The bitrate of the file. 
            magma_file.data["audio"]["bitrate-kbps"] = audioprops.bitrate
            automatically_generated_tags.append(("bitrate-kbps", magma_file.data["audio"]["bitrate-kbps"]))
        except: pass
        try: 
            # The sampling rate. 
            magma_file.data["audio"]["samplerate-Hz"] = audioprops.sampleRate
            automatically_generated_tags.append(("samplerate-Hz", magma_file.data["audio"]["samplerate-Hz"]))
        except: pass
        try: 
            # And the raw filesize. 
            magma_file.data["size-Bytes"] = tagfile.length()
            automatically_generated_tags.append(("size-Bytes", magma_file.data["size-bits"]))
        except: pass
    except: 
        pass
    
    print "The following tags were added:"
    for i in automatically_generated_tags: 
        print i
    # TODO: Ask the user which of the tags to add. 
    
    magma_file.data["title"] = raw_input("Please enter the title of the file: ")
    magma_file.data["description"] = raw_input("Please enter a description of the file: ")
    
    # Now add one of the artists. 
    magma_file.data["audio"]["artists"]  = []
    artist1 = {}
    artist1["name"] = raw_input("Please enter the name of its artist: ")
    artist1["website"] = raw_input("Please enter an url to the website of the artist: ")
    magma_file.data["audio"]["artists"] .append(artist1)
    
    # And add some more data. 
    magma_file.data["audio"]["album"] = raw_input("Please enter the name of the album: ")
    magma_file.data["year"] = raw_input("Please enter the year when this work was published: ")
    magma_file.data["audio"]["tracknumber"] = raw_input("Please enter the tracknumber of this file: ")
    



def create_magma_list_interactively(files):
    """Create magma files interactively via commandline dialogs.
    
    @param files: A list of files to hash.
    @type files: List
    """
    magma = Magma(input_files=files)
    print "Basic magma list: "
    print magma
    
    ask_audio(files, magma)
    
    


def usage(): 
    """Usage instructions.
    
    @return: Usage instructions (String)."""
    # Just use the docstring. 
    usage = __doc__.split("\n\n")[1:]
    usage_string = "\n\n".join(usage)
    return usage_string

#### Self-Test ####

def _test(): 
    """Do all doctests.
    
    @return: None"""
    from doctest import testmod
    testmod()

# If this gets called automatically, load the magma list and print the magnets, or create a magma list. 
if __name__ == "__main__": 
    _test()
    from sys import argv
    create_magma_list_interactively(argv[1:])



#!/usr/bin/env python
# encoding: utf-8

# MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>


"""Features which should be available in the future, but aren't yet well tested. 

New Features will first be implemented here and moved to the main part of the module as soon as they are ready (to give you a bit of shielding from the mistakes we're bound to make). 

You shouldn't count on features from __feature__ to come into the present whatever happens. Some might just have been the wrong approach, and others might prove too bugprone or similar. 

"""

all_feature_names = []

# Import the classes we want to improve. 
try: 
    from magma import Magma as CurrentMagma
except: 
    from magma_list import Magma as CurrentMagma
try: 
    from magma.magma_list import MagmaFile as CurrentMagmaFile
except: 
    from magma_list import MagmaFile as CurrentMagmaFile


# First subclass Magma, and initialize it. 
# It is the main class, and most functionality should go here. 
class Magma(CurrentMagma):
    def __init__(self, *args, **kwds):
        super(Magma, self).__init__(*args, **kwds)

# Then also subclass MagmaFile, and initialize it. 
class MagmaFile(CurrentMagmaFile):
    def __init__(self, *args, **kwds):
        super(MagmaFile, self).__init__(*args, **kwds)



#!/usr/bin/env python
# encoding: utf-8

# MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 
# 
# Copyright © 2008 Arne Babenhauserheide
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>


"""MAGnet MAnifest Management - Readout and create lists of magnets in yaml format. 

Magma lists are lists of files which can be downloaded via magnet links. 

They are written in yaml format to be easily readable as well as flexible and powerful. 

You can install this script via setuptools: 

$ easy_install magma

!!! IN PLANNING PHASE !!!

This script parses the basic features of MAGMAv0.4, which is being specified at the moment. 
It doesn't parse the current MAGMAv0.2 files!

Use it only, if you want to take a peek at what the updated specification can bring. 

It depends on 
 - pyyaml: http://pyyaml.org/. 
 - tagpy

The API documentation is available from http://gnuticles.gnufu.net/pymagma/, 
and the code is available from a Mercurial repository: http://freehg.org/u/ArneBab/magma/. 

Changes: 

0.3.9: 

- Added script to create magma lists for phex releases: create_phex_release_magmas.py

0.3.8: 

- No longer depend on IPy - don't check alt-loc IP:PORT, so other settings can be included, like i2p addresses and similar. No use restricting users when it doesn't gain much.
- Updated usage information and instruction for adding alt-locs in magma_creator_cli.py
- Updated usage information for sha1_gnutelly.py
- A bit nicer API documentation. 

0.3.7: 

- Added script sha1_gnutella.py, which calculates the sha1 hash in Gnutella style (encoded BASE32 uppercase) for all input files.

0.3.6: 

- Added alt-locs to the magnets of the files and to the v0.2 compatibility magnet list.

0.3.5: 

- Added compatibility section, so the magnet links can also be read out 
  by programs supporting only Magmav0.2
- Remove empty items in all listed files before saving, so they are more readable. 
- FIX: Magma module wasn't included in the egg. 

0.3.4: 

- FIX: API was only callable via magma.magma.* - now you can just call "import magma" again. .

0.3.3: 

- FIX: Import error - only imported from modules in the development dir, not from installed ones.

0.3.2: 

- FIX: magma_list.py had wrong header line (tried to open with 'pythons').

0.3.1: 

- Added support for multiple substitude urls and altlocs. 

0.3.0: 

- Added ebuild_creator_cli.py as script. 

0.2.9: 

- Added (mostly fake) ebuild_creator_cli as proof of concept. 
- Changed "name" parameter in files to "filename" to avoid ambiguity (you remember, that this IS in planning phase). 

0.2.8: 

- FIX: Parsing failed, when passed invalid lists. 

0.2.7: 

- Added return values to all method and function docstrings. 

0.2.6: 

- Added docstrings to all functions and classes. 

0.2.5: 

- Added load and dump function to work with magma more easily: 
    - magma.load(filedata) returns a Magma object with all the attributes like magma.magnets, magma.files and magma.metadata . 
    - magma.dump(Magma object) returns its string presentation for saving in a file. 
- Name of the file list parameter and of the URN parameter can now be changed easily. 
- Some cleaning up. 

0.2.4: 

- FIX: Header missed the #

0.2.3: 

- FIX: Saved magma list missed the magma header. 

0.2.2: 

- magma_list.py magma_from_files TARGET file1, file2, ... now works. 

0.2.1: 

- names in the magma list no longer contain the path to the file. 
- Improved documentation. 

0.2: 

- Uses setuptools. 
- Adds Usage info. 
- magma_list.py now callable from commandline. 
- Documentation improvements and API docs. 
- Multiple file magma creation. 

0.1: 

- Creates and reads Magma lists. 

"""

from setuptools import setup, find_packages
from magma import __version__, __author__, __depends__

# Create the desription from the docstrings 

DESCRIPTION = __doc__.split("\n")[0]

LONG_DESCRIPTION = "\n".join(__doc__.split("\n")[1:])

DEPENDENCIES = __depends__.split(", ") # not yet used, because it is too extensive. 
# See install_requires in setup. 

setup(name='magma',
      version=__version__,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION, 
      author=__author__,
      author_email='arne_bab@web.de',
      keywords=["magma", "yaml", "p2p", "magnet", "gnutella"], 
      license="GNU GPL-3 or later", 
      platforms=["any"], 
      install_requires = ["pyyaml", "tagpy"], 
      classifiers = [
            "License :: OSI Approved :: GNU General Public License (GPL)",
            "Programming Language :: Python",
            "Operating System :: OS Independent",
            "Topic :: Software Development :: Libraries :: Python Modules",
            "Topic :: Communications :: File Sharing", 
            "Topic :: Communications :: File Sharing :: Gnutella", 
            "Intended Audience :: Developers", 
            "Intended Audience :: End Users/Desktop", 
            "Environment :: Console", 
            "Development Status :: 1 - Planning", 
            #"Development Status :: 3 - Alpha"
            ],
      packages = find_packages('.'), 
      url='http://freehg.org/u/ArneBab/magma',
      #py_modules=["magma/magma" ],
      scripts=["magma/magma_list.py", "magma/magma_creator_cli.py", "magma/sha1_gnutella.py", "magma/create_phex_release_magmas.py"], 
     )
